#include <stdio.h>
#include "hashtbl.h"

/* Q15
   @requires stock and cart valid addresses,
             *stock and *cart initialized with the same size
   @assigns *stock and *cart
   @ensures add one item with reference ref into the cart,
            provided it is present in the stock
	    update the stock accordingly 
	    returns 1 if it was possible, 0 otherwise */
int add_one_to_cart(hashtbl *stock, hashtbl *cart, int ref) {
  int nb_s = hashtbl_find(ref, *stock);
  if (nb_s == -1 || nb_s == 0) // no association, or association with 0
    return 0;  // no stock
  int nb_c = hashtbl_find(ref, *cart);
  if (nb_c == -1) nb_c = 0;
  hashtbl_replace(ref, nb_c + 1, cart);
  hashtbl_replace(ref, nb_s - 1, stock);
  return 1;
}

/* Q16
   @requires stock and cart valid addresses,
             *stock and *cart initialized with the same size
   @assigns *stock and *cart
   @ensures suppress all items of reference ref from the cart
            and put them back in the stock 
 */
void suppress_article(hashtbl *stock, hashtbl *cart, int ref) {
  int nb_c = hashtbl_find(ref, *cart);
  if (nb_c == -1 || nb_c == 0) 
    return; // nothing to remove
  hashtbl_remove(ref, cart);
  int nb_s = hashtbl_find(ref, *stock);
  if (nb_s == -1) nb_s = 0;
  hashtbl_replace(ref, nb_s + nb_c, stock);
}

/* Q17
   @requires cart initialized
   @assigns nothing
   @ensures print the number of item of reference ref in the cart
 */
void print_cart_article(hashtbl cart, int ref) {
  int nb_c = hashtbl_find(ref, cart);
  if (nb_c == -1)
    printf("Ref. 0x%X:\t0 item\n", ref);
  else
    printf("Ref. 0x%X:\t%d item(s)\n", ref, nb_c);
}
  
/* Q18 */
int main() {
  hashtbl s, c;
  hashtbl_init(&s, 2); //small value for N to test collisions
  hashtbl_init(&c, 2);
  hashtbl_add(0xC0FFEE, 4, &s);
  hashtbl_add(0xC0C0A, 2, &s); 
  hashtbl_add(0xDECAF, 3, &s); 
  hashtbl_add(0xBA0BAB, 1, &s);
  if (!add_one_to_cart(&s, &c, 0xC0FFEE))
    printf("Could not add 0x%X to cart.\n", 0xC0FFEE);
  if (!add_one_to_cart(&s, &c, 0xC0FFEE))
    printf("Could not add 0x%X to cart.\n", 0xC0FFEE);
  if (!add_one_to_cart(&s, &c, 0xDECAF))
    printf("Could not add 0x%X to cart.\n", 0xDECAF);
  if (!add_one_to_cart(&s, &c, 0xBA0BAB))
    printf("Could not add 0x%X to cart.\n", 0xBA0BAB);
  if (!add_one_to_cart(&s, &c, 0xBA0BAB))
    printf("Could not add 0x%X to cart.\n", 0xBA0BAB);
  suppress_article(&s, &c, 0xC0FFEE);
  print_cart_article(c, 0xC0FFEE);
  print_cart_article(c, 0xC0C0A);
  print_cart_article(c, 0xDECAF);
  print_cart_article(c, 0xBA0BAB);
  return 0;
}
