#ifndef _HASHTBL_H
#define _HASHTBL_H

/* Q1: Concrete types for keys and values */
typedef int key;
typedef int value;


/* Q2: Abstract type for the hash tables */
typedef void *hashtbl;

/* Q3
   @requires ph valid address
   @assigns *ph
   @ensures initialize *ph with size N */
void hashtbl_init(hashtbl *ph, int N);


/* Q4
   @requires ph valid address
   @assigns *ph
   @ensures add k |-> v into *ph */
void hashtbl_add(key k, value v, hashtbl *ph);

/* Q5
   @requires nothing
   @assigns nothing
   @ensures returns the last value associated to k in h
            or -1 if there is no such value */
value hashtbl_find(key k, hashtbl h);

/* Q6
   @requires ph valid address
   @assigns *ph
   @ensures remove the last association with key k from *ph 
            no effect if such association does not exist */
void hashtbl_remove(key k, hashtbl *ph);


/* Q7
   @requires ph valid address
   @assigns *ph
   @ensures replace the last association with key k from *ph 
            with the association k |-> v
            add the association if such association did not exist */
void hashtbl_replace(key k, value v, hashtbl *ph);

#endif
